#ifndef PYTHONIC_INCLUDE_ITERTOOLS_COMMON_HPP
#define PYTHONIC_INCLUDE_ITERTOOLS_COMMON_HPP

PYTHONIC_NS_BEGIN

namespace itertools
{

  struct npos {
  };
} // namespace itertools
PYTHONIC_NS_END

#endif
